@@ -156,23 +156,25 @@ class Preprocessor(BasePreprocessor):
                             continue
         self._suite_ids -= self._exclude_suite_ids
 
-        if not self._section_ids:
-            for suite in project_suites:
-                if suite['id'] in self._suite_ids:
-                    suite_sections = self._client.send_get('get_sections/%s&suite_id=%s' % (self._project_id, suite['id']))
-                    for section in suite_sections:    
-                        self._section_ids.add(section['id'])
-        next_iteration = True  # collect all child subsections for sections specified
-        while next_iteration:
-            next_iteration = False
-            for suite in project_suites:
-                if suite['id'] in self._suite_ids:
-                    suite_sections = self._client.send_get('get_sections/%s&suite_id=%s' % (self._project_id, suite['id']))
+        if self._section_ids:
+            selected_sections = True
+        else:
+            selected_sections = False
+
+        for suite in project_suites:
+            if suite['id'] in self._suite_ids:
+                suite_sections = self._client.send_get('get_sections/%s&suite_id=%s' % (self._project_id, suite['id']))
+                next_iteration = True
+                while next_iteration:
+                    next_iteration = False
                     for section in suite_sections:
-                        if section['id'] not in self._section_ids and section['parent_id'] in self._section_ids:    
+                        common_condition = section['id'] not in self._section_ids and section['id'] not in self._exclude_section_ids
+                        if not selected_sections and common_condition and (section['parent_id'] in self._section_ids or not section['parent_id']):
+                            self._section_ids.add(section['id'])
+                            next_iteration = True
+                        if selected_sections and common_condition and section['parent_id'] in self._section_ids:
                             self._section_ids.add(section['id'])
                             next_iteration = True
-        self._section_ids -= self._exclude_section_ids
 
 
     def _collect_cases(self, project_suites):
@@ -215,11 +217,7 @@ class Preprocessor(BasePreprocessor):
                 for i in range(section['depth']):
                     title_level_up += '#'
 
-                section_name = re.sub("[0-9].*\.", "", section['name']).strip()  # Remove numbers in section headers
-
-                if section_name.isupper():  # Remove headers capitalization
-                    section_name = section_name.lower()
-                    section_name = section_name[0].upper() + section_name[1:]
+                section_name = section['name'].strip()
 
                 self._test_cases.append('##%s %s\n\n' % (title_level_up,
                                 section_name))
@@ -255,12 +253,10 @@ class Preprocessor(BasePreprocessor):
         priority_cond = True
         exclude_cond = True
 
-        if type(case[self._params['multi_param_sys_name']]) == int:
-            int_to_list = []
-            int_to_list.append(case[self._params['multi_param_sys_name']])
-            case[self._params['multi_param_sys_name']] = int_to_list
-
         if self._params['multi_param_sys_name'] in case.keys():
+            if type(case[self._params['multi_param_sys_name']]) == int:
+                case[self._params['multi_param_sys_name']] = [case[self._params['multi_param_sys_name']]]
+
             if self._params['multi_param_sys_name'] not in case.keys() and not self._params['add_cases_without_multi_param']:
                     multi_param_cond = False
             elif self._params['multi_param_select_type'] == 'any' and not (set(case[self._params['multi_param_sys_name']]) & self._params['multi_param_matches']):
@@ -289,6 +285,10 @@ class Preprocessor(BasePreprocessor):
         return take_case
 
 
+    def _get_case_data(self, case_id):
+        return self._client.send_get('get_case/%s&case_id' % (case_id))
+
+
     def _collect_case_data(self, suite_id, section_id, title_level_up):
         section_cases = self._client.send_get(
             'get_cases/%s&suite_id=%s&section_id=%s' %
@@ -309,7 +309,7 @@ class Preprocessor(BasePreprocessor):
                 if self._params['add_priority_to_case_header']:
                     header_ending_list.append(self._priorities[case['priority_id']])
                 text = ''
-                if self._params['multi_param_name']:
+                if self._params['multi_param_name'] and self._params['multi_param_sys_name'] in case.keys():
                     for item in case[self._params['multi_param_sys_name']]:
                         text += self._params['multi_param_values'][item] + ', '
                 if self._params['add_multi_param_to_std_table']:
@@ -335,13 +335,16 @@ class Preprocessor(BasePreprocessor):
                 case_template = '/'.join((str(self.project_path), self._template_folder, ''.join((str(case['template_id']), '.j2'))))
 
                 if not os.path.isfile(case_template):
-                    print(f"\n\nhere is no jinja template for test case template_id {case['template_id']} (case_id {case['id']}) in folder {self._template_folder}")
+                    print(f"\n\nThere is no jinja template for test case template_id {case['template_id']} (case_id {case['id']}) in folder {self._template_folder}")
+                    if self._print_case_structure:
+                        print('\nCase structure:')
+                        pprint(case)
                 else:
                     try:
                         template = self._env.get_template(case_template)
-                        result = template.render(case=case, params=self._params).split('\r\n')
+                        result = template.render(case=case, params=self._params).split('\n')
                     except Exception as exception:
-                        print(f"\nThere is problem with jinja template for test case template_id {case['template_id']} (case_id {case['id']}) in folder {self._template_folder}:\n{exception}")
+                        print(f"\nThere is a problem with jinja template for test case template_id {case['template_id']} (case_id {case['id']}) in folder {self._template_folder}:\n{exception}")
                         if self._print_case_structure:
                             print('\nCase structure:')
                             pprint(case)
@@ -349,7 +352,7 @@ class Preprocessor(BasePreprocessor):
 
                     if result:
                         for string in result:
-                            self._test_cases.append(string)
+                            self._test_cases.append(string.strip())
                             self._test_cases.append('\n')
 
 
@@ -379,7 +382,12 @@ class Preprocessor(BasePreprocessor):
                         for string in range(string_counter):
                             self._test_cases.pop(index)
                         if len(self._test_cases) > 0:
+                            renumber_strings = False
+                            if self._std_table[len(self._std_table)-remove_from_case_list].split('|')[1].strip():
+                                renumber_strings = True
                             self._std_table.pop(len(self._std_table)-remove_from_case_list)
+                            if renumber_strings:
+                                self._renumber_strings(remove_from_case_list-1)
                         remove_from_case_list -= 1
 
                         next_iteration = True
@@ -389,6 +397,16 @@ class Preprocessor(BasePreprocessor):
                     string_counter = 0
 
 
+    def _renumber_strings(self, shift):
+        for string_number in range(len(self._std_table)-shift, len(self._std_table)):
+            case_number = self._std_table[string_number].split('|')[1].strip()
+            if case_number:
+                new_number = str(int(case_number) - 1)
+                if len(new_number)<len(case_number):
+                    new_number += ' '
+                self._std_table[string_number] = self._std_table[string_number].replace(case_number, new_number, 1)
+
+
     def _make_std_table_first_row(self, add_column_headers):
         first_row = '| '
         for i, header in enumerate(self._std_table_column_headers):
